@@ -31,7 +31,10 @@ from pulpcore.plugin.stages import (
     Stage,
 )
 
-from pulp_2to3_migration.app.constants import NOT_USED
+from pulp_2to3_migration.app.constants import (
+    DEFAULT_BATCH_SIZE,
+    NOT_USED,
+)
 from pulp_2to3_migration.app.models import (
     Pulp2Importer,
     Pulp2LazyCatalog,
@@ -221,7 +224,6 @@ class ContentMigrationFirstStage(Stage):
             return pulp2importer.pulp3_remote
 
         futures = []
-        batch_size = 1000
         is_lazy_type = content_type in self.migrator.lazy_types
         is_artifactless_type = content_type in self.migrator.artifactless_types
         has_future = content_type in self.migrator.future_types
@@ -271,7 +273,9 @@ class ContentMigrationFirstStage(Stage):
                 if is_lazy_type:
                     # get all Lazy Catalog Entries (LCEs) for this content
                     pulp2lazycatalog = Pulp2LazyCatalog.objects.filter(
-                        pulp2_unit_id=pulp2content.pulp2_id, is_migrated=False)
+                        pulp2_unit_id=pulp2content.pulp2_id,
+                        is_migrated=False,
+                    )
 
                 if is_lazy_type and not pulp2content.downloaded and not pulp2lazycatalog:
                     _logger.warn(_('On_demand content cannot be migrated without an entry in the '
@@ -283,6 +287,8 @@ class ContentMigrationFirstStage(Stage):
                     # find already created pulp3 content
                     pulp3content = pulp2content.pulp3_content
                     extra_info = None
+                    if is_multi_artifact:
+                        extra_info = pulp_2to3_detail_content.get_treeinfo_serialized()
 
                 else:
                     # create pulp3 content and assign relations if present
@@ -308,8 +314,14 @@ class ContentMigrationFirstStage(Stage):
 
                         lces = pulp2lazycatalog.filter(pulp2_storage_path=image_path)
                         if lces:
+                            remote_declarative_artifacts = []
+
                             for lce in lces:
                                 remote = get_remote_by_importer_id(lce.pulp2_importer_id)
+
+                                if not remote and not downloaded:
+                                    continue
+
                                 remotes.add(remote)
                                 da = DeclarativeArtifact(
                                     artifact=artifact,
@@ -317,7 +329,16 @@ class ContentMigrationFirstStage(Stage):
                                     relative_path=image_relative_path,
                                     remote=remote,
                                     deferred_download=not downloaded)
-                                d_artifacts.append(da)
+                                remote_declarative_artifacts.append(da)
+
+                            if not remote_declarative_artifacts:
+                                _logger.warn(_(
+                                    'On_demand content cannot be migrated without a remote '
+                                    'pulp2 unit_id: {}'.format(pulp2content.pulp2_id))
+                                )
+                                continue
+
+                            d_artifacts.extend(remote_declarative_artifacts)
                         else:
                             da = DeclarativeArtifact(
                                 artifact=artifact,
@@ -381,14 +402,11 @@ class ContentMigrationFirstStage(Stage):
                         # To create multiple remote artifacts, create multiple instances of
                         # declarative content which will differ by url/remote in their
                         # declarative artifacts
+                        at_least_one_lce_migrated = False
                         for lce in pulp2lazycatalog:
                             remote = get_remote_by_importer_id(lce.pulp2_importer_id)
                             deferred_download = not pulp2content.downloaded
                             if not remote and deferred_download:
-                                _logger.warn(_(
-                                    'On_demand content cannot be migrated without a remote '
-                                    'pulp2 unit_id: {}'.format(pulp2content.pulp2_id))
-                                )
                                 continue
 
                             relative_path = (
@@ -401,9 +419,14 @@ class ContentMigrationFirstStage(Stage):
                                 remote=remote,
                                 deferred_download=deferred_download)
                             lce.is_migrated = True
+                            at_least_one_lce_migrated = True
                             dc = DeclarativeContent(content=pulp3content, d_artifacts=[da])
                             dc.extra_data = future_relations
                             await self.put(dc)
+
+                        if not at_least_one_lce_migrated:
+                            _logger.warn(_('On_demand content cannot be migrated without a remote '
+                                           'pulp2 unit_id: {}'.format(pulp2content.pulp2_id)))
                         future_relations.update({'lces': list(pulp2lazycatalog)})
                     else:
                         relative_path = (
@@ -424,7 +447,7 @@ class ContentMigrationFirstStage(Stage):
 
                 if has_future and dc:
                     futures.append(dc)
-                resolve_futures = len(futures) >= batch_size or pb.done == pb.total
+                resolve_futures = len(futures) >= DEFAULT_BATCH_SIZE or pb.done == pb.total
                 if resolve_futures:
                     for dc in futures:
                         await dc.resolution()
@@ -450,7 +473,7 @@ class UpdateLCEs(Stage):
 
                 Pulp2LazyCatalog.objects.bulk_update(objs=pulp2lces_batch,
                                                      fields=['is_migrated'],
-                                                     batch_size=1000)
+                                                     batch_size=DEFAULT_BATCH_SIZE)
 
             for d_content in batch:
                 await self.put(d_content)
@@ -480,7 +503,7 @@ class RelatePulp2to3Content(Stage):
 
                 pulp2content.__class__.objects.bulk_update(objs=pulp2content_batch,
                                                            fields=['pulp3_content'],
-                                                           batch_size=1000)
+                                                           batch_size=DEFAULT_BATCH_SIZE)
 
             for d_content in batch:
                 await self.put(d_content)
