@@ -14,6 +14,7 @@ from pulpcore.plugin.models import (
     Publication,
     ProgressReport,
 )
+from pulp_2to3_migration.app.constants import DEFAULT_BATCH_SIZE
 from pulp_2to3_migration.app.models import (
     Pulp2Content,
     Pulp2Distributor,
@@ -74,7 +75,7 @@ def pre_migrate_content(content_model, mutable_type, lazy_type, premigrate_hook)
         content_model: Models for content which is being migrated.
         mutable_type: Boolean that indicates whether the content type is mutable.
     """
-    batch_size = 1000
+    batch_size = 500
     content_type = content_model.pulp2.TYPE_ID
     pulp2content = []
     pulp2mutatedcontent = []
@@ -425,13 +426,8 @@ def pre_migrate_importer(repo_id, importers, importer_types, repo=None):
             if importer.pulp2_config.get('feed') != importer_data.config.get('feed'):
                 importer.pulp3_remote.delete()
                 importer.pulp3_remote = None
-                # find LCEs
-                pulp2lazycatalog = Pulp2LazyCatalog.objects.filter(
-                    pulp2_importer_id=importer.pulp2_object_id)
-                for lce in pulp2lazycatalog:
-                    lce.is_migrated = False
-                Pulp2LazyCatalog.objects.bulk_update(objs=pulp2lazycatalog,
-                                                     fields=['is_migrated'])
+                # do not flip is_migrated to False for LCE for at least once migrated importer
+
             importer.pulp2_last_updated = last_updated
             importer.pulp2_config = importer_data.config
             importer.is_migrated = False
@@ -539,7 +535,7 @@ def pre_migrate_repocontent(repo):
                                 pulp2_repository=repo)
         repocontent.append(item)
 
-    Pulp2RepoContent.objects.bulk_create(repocontent)
+    Pulp2RepoContent.objects.bulk_create(repocontent, batch_size=DEFAULT_BATCH_SIZE)
 
 
 def mark_removed_resources(plan, type_to_repo_ids):
@@ -578,7 +574,7 @@ def mark_removed_resources(plan, type_to_repo_ids):
 
         Pulp2Repository.objects.bulk_update(objs=removed_repos,
                                             fields=['not_in_plan'],
-                                            batch_size=1000)
+                                            batch_size=DEFAULT_BATCH_SIZE)
 
         # Mark importers
         mongo_imp_object_ids = set(str(i.id) for i in Importer.objects.only('id'))
@@ -596,7 +592,7 @@ def mark_removed_resources(plan, type_to_repo_ids):
 
         Pulp2Importer.objects.bulk_update(objs=removed_imps,
                                           fields=['not_in_plan'],
-                                          batch_size=1000)
+                                          batch_size=DEFAULT_BATCH_SIZE)
 
         # Mark distributors
         mongo_dist_object_ids = set(str(i.id) for i in Distributor.objects.only('id'))
@@ -614,7 +610,7 @@ def mark_removed_resources(plan, type_to_repo_ids):
 
         Pulp2Distributor.objects.bulk_update(objs=removed_dists,
                                              fields=['not_in_plan'],
-                                             batch_size=1000)
+                                             batch_size=DEFAULT_BATCH_SIZE)
 
 
 def delete_old_resources(plan):
