@@ -1,6 +1,6 @@
 import json
 
-from drf_spectacular.utils import extend_schema_serializer
+from drf_spectacular.utils import extend_schema_field
 from gettext import gettext as _
 from django.urls import reverse
 from jsonschema import Draft7Validator
@@ -143,7 +143,7 @@ class Pulp2ContentSerializer(ModelSerializer):
 
     pulp3_repository_version = serializers.SerializerMethodField(read_only=True)
 
-    @extend_schema_serializer(many=False)
+    @extend_schema_field(field=serializers.CharField)
     def get_pulp3_repository_version(self, obj):
         """
         Get pulp3_repository_version href from pulp2repo if available
@@ -195,7 +195,7 @@ class Pulp2RepositoriesSerializer(ModelSerializer):
     pulp3_distribution_hrefs = serializers.SerializerMethodField(read_only=True)
     pulp3_repository_href = serializers.SerializerMethodField(read_only=True)
 
-    @extend_schema_serializer(many=False)
+    @extend_schema_field(field=serializers.CharField)
     def get_pulp3_repository_href(self, obj):
         """
         Get pulp3_repository_href from pulp2repo
@@ -204,7 +204,7 @@ class Pulp2RepositoriesSerializer(ModelSerializer):
         if rv:
             return get_pulp_href(rv.repository)
 
-    @extend_schema_serializer(many=False)
+    @extend_schema_field(field=serializers.CharField)
     def get_pulp3_remote_href(self, obj):
         """
         Get pulp3_remote_href from pulp2repo
@@ -213,7 +213,7 @@ class Pulp2RepositoriesSerializer(ModelSerializer):
         if remote:
             return get_pulp_href(remote)
 
-    @extend_schema_serializer(many=False)
+    @extend_schema_field(field=serializers.CharField)
     def get_pulp3_publication_href(self, obj):
         """
         Get pulp3_publication_href from pulp3_repository_version
@@ -222,7 +222,7 @@ class Pulp2RepositoriesSerializer(ModelSerializer):
         if rv:
             return get_pulp_href(rv.publication_set.first())
 
-    @extend_schema_serializer(many=True)
+    @extend_schema_field(field=serializers.ListField(child=serializers.CharField()))
     def get_pulp3_distribution_hrefs(self, obj):
         """
         Get pulp3_distribution_hrefs from pulp3_repository_version
